package com.autentia.tutoriales.interprete;

import java.math.BigDecimal;
import java.util.Stack;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.math.NumberUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;


@Service
public class InterpreterFunction implements ExpressionFunction {

	private ExpressionFunction expressionsTree;
	
	private Stack<ExpressionFunction> expressionsStack;
	
	private String actualExpression;
	
	public String getActualExpression() {
		return actualExpression;
	}

	@Autowired
	public InterpreterFunction(@Value("#{props['interpreter.expression']}")String expression) {
		this.actualExpression = expression;
		expressionsStack = new Stack<ExpressionFunction>();
		
		for (String token : expression.split(" ")) {
			ExpressionFunction subFunction  = createTheRightFunction(token);					
			expressionsStack.push(subFunction);
		}
		expressionsTree = expressionsStack.pop();
	}
	
	//TODO considerar extraer esto a una Factora de familias de funciones. Para una sesin de refactorizacin
	private ExpressionFunction createTheRightFunction(String token) {	
		
		ExpressionFunction funcion = null;
		
		if (NumberUtils.isNumber(token)) {
			funcion = new NumberFunction(new BigDecimal(token));
		} else if(StringUtils.isAlpha(token)) {
			funcion = new VariableFunction(token);
		} else if ("+".equals(token)) {
			funcion = new SumFunction(expressionsStack.pop(),expressionsStack.pop());
		} else if ("-".equals(token)) {
			ExpressionFunction right = expressionsStack.pop();
			ExpressionFunction left = expressionsStack.pop();
			funcion = new SubtractFunction(left, right);
		} else if ("*".equals(token)) {
			funcion = new MultFunction(expressionsStack.pop(), expressionsStack.pop());
		} else if ("/".equals(token)) {
			ExpressionFunction right = expressionsStack.pop();
			ExpressionFunction left = expressionsStack.pop();
			funcion =  new DivFunction(left, right);
		} else if ("@".equals(token)) {
			funcion = new LogeFunction(expressionsStack.pop());
		} else {
			throw new ExpressionFunctionException("regla no reconocida: " + token);
		}
		
		
		return funcion;
	}

	public BigDecimal evaluate(ExpressionFunctionContext context) {
		return expressionsTree.evaluate(context);
	}

}
